c     Copyright (C)
c     2008-2013 Universit Paul Sabatier
c     2008-2013 Centre National de la Recherche Scientifique
c     2008-2013 Ecole des Mines d'Albi
c     2008-2013 Universit de Bordeaux 1
c     2008-2013 Fondation STAE
c     2014-2018 Meso-Star
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c     
      double precision function Planck(T,nu1,nu2)
      implicit none
      double precision T,lambda_c,nu1,nu2,nu_c,B
      double precision lambda1,lambda2,dlambda,dnu
c
c     Purpose: to compute the mean value of the Planck intensity (W/m/sr/cm)
c     over a given spectral interval
c
c     Inputs:
c       + T: temperature (K)
c       + nu1: wavenumber at the beginning of the spectral interval (cm)
c       + nu2: wavenumber at the end of the spectral interval (cm)
c
c     Output: Planck (W/m/sr/cm)
c

      dnu=nu2-nu1
c     Debug
      if (dnu.le.0.0D+0) then
         write(*,*) 'Error from routine Planck:'
         write(*,*) 'dnu=',dnu
         stop
      endif
c     Debug
      call inversecm2micrometer(nu1,lambda1)
      call inversecm2micrometer(nu2,lambda2)
      dlambda=lambda1-lambda2
c     Debug
      if (dlambda.le.0.0D+0) then
         write(*,*) 'Error from routine Planck:'
         write(*,*) 'dlambda=',dlambda
         stop
      endif
c     Debug
      nu_c=(nu1+nu2)/2.0D+0 ! cm
      call inversecm2micrometer(nu_c,lambda_c)

      Planck=B(T,lambda_c) ! W/m/sr/m
      Planck=Planck*dlambda/dnu ! W/m/sr/cm

      return
      end




      function B(T,lambda)
      implicit none
c     
c     Planck intensity (W/m/sr/m) as function of wavelenght (m) and temperature (K)
c     
c     Inputs:
c       + T: temperature in K
c       + lambda: wavelenght (m)
c
c     Output: B (W/m/sr/m)
c
      double precision B,T,lambda
      double precision c1,c2,in
      include 'parameters.inc'

      c1=2*pi*hPl*c0**2 ! W.m
      c2=hPl*c0/kBz*1.D+6 ! m.K
      in=lambda*T
      if (T.eq.0) then
         B=0.0D+0
      else
         B=c1/(in**5*(dexp(c2/in)-1))*T**5*1.D+24/pi ! W/m/sr/m
      endif

      return
      end


      subroutine inversecm2micrometer(nu,lambda)
      implicit none
c     Purpose: to convert a wavenumber (in cm) to a wavelenght (in m)
      double precision nu,lambda

      lambda=1.0D+4/nu ! m

      return
      end


      subroutine micrometer2inversecm(lambda,nu)
      implicit none
c     Purpose: to converta wavelenght (in m)  to a wavenumber (in cm) 
      double precision nu,lambda

      nu=1.0D+4/lambda ! cm

      return
      end
